@@ -16,14 +16,19 @@ Classo and pathlasso are the main functions, they can call every algorithm acord
 
 def Classo(matrix,lam,typ = 'R1', meth='DR', rho = 1.345, get_lambdamax = False, true_lam=False, e=None, rho_classification=-1., w = None, intercept = False):
 
-    if not w is None : matrices = (  matrix[0]/w, matrix[1],matrix[2] )
+    if not w is None : matrices = (  matrix[0]/w, matrix[1]/w,matrix[2] )
     else : matrices = matrix
-    if intercept : 
-        means = (np.mean(matrices[0],axis=0), np.mean(matrices[2]) )
-        matrices = (matrices[0]-means[0],matrices[1],matrices[2]-means[1])
 
+    X,C,y = matrices
     
+
     if(typ=='R3'):
+        if intercept :         
+            # here we use the fact that for R1 and R3, the intercept is simple beta0 = ybar-Xbar .vdot(beta) so by changing the X to X-Xbar and y to y-ybar we can solve standard problem
+            Xbar, ybar = np.mean(X,axis=0), np.mean(y)  
+            matrices = (X-Xbar,C,y-ybar)
+
+
         if not meth in ['Path-Alg', 'DR']: meth='DR'
         if e is None or e == len(matrices[0])/2: 
             r = 1.
@@ -36,7 +41,17 @@ def Classo(matrix,lam,typ = 'R1', meth='DR', rho = 1.345, get_lambdamax = False,
         else : beta, s = Classo_R3(pb, lam)
         s = s/np.sqrt(e)
 
+        if intercept : 
+            betaO = ybar - np.vdot(Xbar,beta)
+            beta = np.array([betaO]+list(beta))
+
+
+
     elif(typ=='R4'):
+
+        if intercept : 
+            raise ValueError("The classo package does not perform formulation R4 (Contrained sparse Huber regression with concomitant scale estimation) with an intercept")
+
         if not meth in ['Path-Alg', 'DR']: meth='DR'
         if e is None or e == len(matrices[0]): 
             r = 1.
@@ -51,19 +66,31 @@ def Classo(matrix,lam,typ = 'R1', meth='DR', rho = 1.345, get_lambdamax = False,
 
 
     elif(typ=='R2'):
+
+        if intercept : 
+            raise ValueError("The classo package does not perform formulation R2 (Contrained sparse Huber regression) with an intercept")
+
         if not meth in ['Path-Alg', 'P-PDS' , 'PF-PDS' , 'DR']: meth = 'ODE'
-        pb = problem_R2(matrices,meth,rho)
+        pb = problem_R2(matrices,meth,rho, intercept=intercept)
         lambdamax = pb.lambdamax
         if (true_lam): beta = Classo_R2(pb,lam/lambdamax)
         else : beta = Classo_R2(pb, lam)
 
     elif (typ == 'C2'):
+
+        if intercept : 
+            raise ValueError("The classo package does not perform classification with an intercept")
+
         lambdamax = h_lambdamax(matrices[0],matrices[2],rho)
         if true_lam : BETA = solve_path(matrices, lam/lambdamax, False, rho_classification, 'huber_cl')[0]
         else : BETA = solve_path(matrices, lam, False, rho_classification, 'huber_cl')[0]
         beta = BETA[-1]
 
     elif (typ == 'C1'):
+
+        if intercept : 
+            raise ValueError("The classo package does not perform classification with an intercept")
+
         lambdamax = h_lambdamax(matrices[0],matrices[2],0)
         if (true_lam): BETA = solve_path(matrices,lam/lambdamax, False,0, 'cl')[0]
         else : BETA = solve_path(matrices,lam, False,0, 'cl')[0]
@@ -71,17 +98,28 @@ def Classo(matrix,lam,typ = 'R1', meth='DR', rho = 1.345, get_lambdamax = False,
 
 
     else: # LS
+        if intercept : 
+            # here we use the fact that for R1 and R3, the intercept is simple beta0 = ybar-Xbar .vdot(beta) so by changing the X to X-Xbar and y to y-ybar we can solve standard problem
+            Xbar, ybar = np.mean(X,axis=0), np.mean(y)
+            matrices = (X-Xbar,C,y-ybar)
+
+
         if not meth in ['Path-Alg', 'P-PDS' , 'PF-PDS' , 'DR']: meth='DR'
         pb = problem_R1(matrices,meth)
         lambdamax = pb.lambdamax
         if (true_lam) : beta = Classo_R1(pb,lam/lambdamax)
         else : beta = Classo_R1(pb,lam)
 
+        if intercept : 
+            betaO = ybar - np.vdot(Xbar,beta)
+            beta = np.array([betaO]+list(beta))
+
+
+    if not w is None : 
+        if intercept : beta[1:] = beta[1:] / w
+        else : beta = beta/w
+        
 
-    if not w is None : beta = beta / w
-    if intercept : 
-        betaO = means[1] - means[0].dot(beta)
-        beta = np.array([betaO]+list(beta))
 
     if (typ  in ['R3','R4']):
         if (get_lambdamax): return(lambdamax,beta,s)
@@ -101,19 +139,24 @@ def pathlasso(matrix,lambdas=False,n_active=0,lamin=1e-2,typ='R1',meth='Path-Alg
     else: lambdass = np.linspace(1.,lamin,80)
 
 
-    if not w is None : matrices = (  matrix[0]/(w+1e-3), matrix[1],matrix[2] )
+    if not w is None : matrices = (  matrix[0]/w, matrix[1]/w,matrix[2] )
     else : matrices = matrix
-    if intercept : 
-        means = (np.mean(matrices[0],axis=0), np.mean(matrices[2]) )
-        matrices = (matrices[0]-means[0],matrices[1],matrices[2]-means[1])
+    
+    X,C,y = matrices
 
     if(typ=='R2'):
+        if intercept : 
+            raise ValueError("The classo package does not perform formulation R2 (Contrained sparse Huber regression) with an intercept")
         pb = problem_R2(matrices,meth,rho)
         lambdamax = pb.lambdamax
         if (true_lam): lambdass=[lamb/lambdamax for lamb in lambdass]
         BETA  = pathlasso_R2(pb,lambdass,n_active=Nactive)
 
     elif(typ=='R3'):
+        if intercept : 
+            # here we use the fact that for R1 and R3, the intercept is simple beta0 = ybar-Xbar .vdot(beta) so by changing the X to X-Xbar and y to y-ybar we can solve standard problem
+            Xbar, ybar = np.mean(X,axis=0), np.mean(y)  
+            matrices = (X-Xbar,C,y-ybar)
         if e is None or e == len(matrices[0])/2: 
             r = 1.
             pb = problem_R3(matrices,meth)
@@ -125,9 +168,13 @@ def pathlasso(matrix,lambdas=False,n_active=0,lamin=1e-2,typ='R1',meth='Path-Alg
         BETA,S = pathlasso_R3(pb,lambdass,n_active=Nactive)
         S=np.array(S)/r**2
         BETA = np.array(BETA)
+        if intercept : 
+            BETA = np.array([ [ybar - Xbar.dot(beta) ]+list(beta) for beta in BETA] )
 
 
     elif(typ=='R4'):
+        if intercept : 
+            raise ValueError("The classo package does not perform formulation R4 (Contrained sparse Huber regression with concomitant scale estimation) with an intercept")
         if e is None or e == len(matrices[0]): 
             r = 1.
             pb = problem_R4(matrices,meth,rho)
@@ -142,33 +189,52 @@ def pathlasso(matrix,lambdas=False,n_active=0,lamin=1e-2,typ='R1',meth='Path-Alg
         BETA = np.array(BETA)
         
     elif(typ == 'C2'):
+        if intercept : 
+            raise ValueError("The classo package does not perform classification with an intercept")
         lambdamax = h_lambdamax(matrices[0],matrices[2],rho)
         if (true_lam): lambdas=[lamb/lambdamax for lamb in lambdass]
         BETA = pathalgo_general(matrices, lambdass, 'huber_cl', n_active=Nactive, rho=rho_classification)
 
     elif (typ == 'C1'):
+        if intercept : 
+            raise ValueError("The classo package does not perform classification with an intercept")
         lambdamax = h_lambdamax(matrices[0],matrices[2],0)
         if (true_lam): lambdass = [lamb / lambdamax for lamb in lambdass]
         BETA = pathalgo_general(matrices, lambdass, 'cl', n_active=Nactive)
 
-    else:
+    else: # R1
+        if intercept : 
+            # here we use the fact that for R1 and R3, the intercept is simple beta0 = ybar-Xbar .vdot(beta) so by changing the X to X-Xbar and y to y-ybar we can solve standard problem
+            Xbar, ybar = np.mean(X,axis=0), np.mean(y)  
+            matrices = (X-Xbar,C,y-ybar)
         pb = problem_R1(matrices,meth)
         lambdamax = pb.lambdamax
         if (true_lam): lambdass=[lamb/lambdamax for lamb in lambdass]
         BETA = pathlasso_R1(pb,lambdass,n_active=n_active)
 
+        if intercept : 
+            BETA = np.array([ [ybar - Xbar.dot(beta) ]+list(beta) for beta in BETA] )
+
     real_path = [lam*lambdamax for lam in lambdass]
 
-    if not w is None : BETA = np.array([beta / (w+1e-3) for beta in BETA])
-    if intercept : 
-        BETA = np.array([ [means[1] - means[0].dot(beta) ]+list(beta) for beta in BETA] )
+    if not w is None : 
+        if intercept : ww = np.array([1]+list(w))
+        else : ww = w
+
+        BETA = np.array([beta / ww for beta in BETA])
 
 
-    if(typ in ['R3','R4'] and return_sigm): return(BETA,real_path,S)
-    return(BETA,real_path)
+    if(typ in ['R3','R4'] and return_sigm): return(np.array(BETA),real_path,S)
+    return(np.array(BETA),real_path)
  
     
 
+
+
+
+
+
+
 '''
 # Cost fucntions for the three 'easiest' problems. Useful for test, to compare two solutions slightly different
 def L_LS(A,y,lamb,x): return(LA.norm( A.dot(x) - y )**2 + lamb * LA.norm(x,1))
