@@ -20,7 +20,7 @@ def Classo_R3(pb,lam):
 
     if (lam == 0.): 
         beta =  unpenalized(pb.matrix)
-        sigma= LA.norm(A.dot(beta)-y)/np.sqrt(pb.e)
+        sigma= LA.norm(A.dot(beta)-y)/np.sqrt(m/2)
         return beta,sigma
 
 
@@ -132,7 +132,6 @@ class problem_R3 :
 
         (m,d,k) = self.dim
         self.weights = np.ones(d)
-        self.e = m/2
         self.tol = 1e-4
           
         self.regpath = False
@@ -142,9 +141,9 @@ class problem_R3 :
         self.proj_sigm = lambda x : max(x,0)
         self.mu = 1.95
 
-        self.gam        = np.sqrt(self.dim[1])
+        self.gam        = np.sqrt(d)
         self.Aty        = (A.T).dot(y)
-        self.sigmax     = LA.norm(y)/np.sqrt(e)
+        self.sigmax     = LA.norm(y)/np.sqrt(m/2)
         self.lambdamax  = 2 * LA.norm(self.Aty, np.infty) / self.sigmax
         self.init       = 1.,1.,np.zeros(m), np.zeros(d), np.zeros(d)    
 
@@ -153,8 +152,8 @@ class problem_R3 :
     def compute_param(self):
         (A,C,y) = self.matrix
         m,d,k = self.dim
-        self.Anorm = LA.norm(A, 2)
-        self.Anorm2 = self.Anorm ** 2
+        self.Anorm = LA.norm(A, 'fro')
+        self.Anorm2 = LA.norm(A, 2) ** 2
         c = d**2/self.Anorm2  # parameter for Concomitant problem : the matrix is scaled as c*A^2
         self.c = c
         self.Q1, self.Q2 = QQ(c, A)
