@@ -65,8 +65,8 @@ class classo_problem:
                                                                 #          mean of initial y )
 
         if not self.formulation.w is None : 
-            if min(self.formulation.w) < 1e-3 : 
-                raise ValueError("w has to be positive weights, here it has a value smaller than 1e-3")
+            if min(self.formulation.w) < 1e-8 : 
+                raise ValueError("w has to be positive weights, here it has a value smaller than 1e-8")
 
         if self.formulation.intercept : 
             data.label = np.array(['intercept']+list(data.label))
@@ -277,8 +277,17 @@ class PATHparameters:
         n_active (int): if it is higher than 0, then the algo stop computing the path when n_active variables are actives. then the solution does not change from this point.
             Dafault value : 0
 
-        lambdas (numpy.ndarray) : list of lambdas for computinf lasso-path for cross validation on lambda.
-            Default value : np.array([10**(-delta * float(i) / Nlam) for i in range(0,Nlam) ] ) with delta=2. and Nlam = 40
+        lambdas (numpy.ndarray) : list of rescaled lambdas for computing lasso-path
+            Default value : None, which means line space between 1 and lamin and Nlam points, with logarithm scale or not depending on logscale
+
+        Nlam (int) : number of points in the lambda-path if lambdas is still None
+            Default value : 80
+
+        lamin (float) : lambda minimum if lambdas is still None
+            Default value : 1e3
+
+        logscale (bool): when lambdas is set to None (default), this parameters tells if it should be set with log scale or not
+            Default value : True
 
         plot_sigma (bool) : if True then the print method of the solution will also show sigma if it is computed (formulation R3 or R4)
             Default value : True
@@ -290,20 +299,28 @@ class PATHparameters:
         self.formulation = 'not specified'
         self.numerical_method = method
         self.n_active = 0
-        lamin= 1e-2
-        Nlam = 40
-        self.lambdas = np.array([10**(np.log10(lamin) * float(i) / (Nlam+1)) for i in range(0,Nlam) ] )
+        self.Nlam = 80
+        self.lamin = 1e3
+        self.logscale = True
+        self.lambdas = None
         self.plot_sigma = True
 
     def __repr__(self): 
-        string  = '\n     numerical_method : ' + str(self.numerical_method)
-        string += '\n     Npath = ' + str(len(self.lambdas))
-        string += '\n     lamin = ' + str(round(self.lambdas[-1],3))
-        string += '\n     lamax = ' + str(round(self.lambdas[0],3))
-        
-        if self.n_active > 0 : 
-            string += '\n     n_active = ' + str(self.n_active)
+        if not self.lambdas is None : 
+            self.Nlam = len(self.lambdas)
+            self.lamin = min(self.lambdas)
+            if self.logscale: typ = " with log-scale"
+            else : typ = " with linear-scale"
+        else :
+            typ = ' the list has been entered manually'
 
+        string  = '\n     numerical_method : ' + str(self.numerical_method)
+        string += '\n     lamin = ' + str(self.lamin)
+        string += '\n     Nlam = ' + str(self.Nlam)
+        string += typ
+        if self.n_active > 0 : 
+            string += '\n     maximum active variables = ' + str(self.n_active)
+        
         return string
                                 
 
@@ -318,8 +335,17 @@ class CVparameters:
             'Path-Alg' (path algorithm) , 'P-PDS' (Projected primal-dual splitting method) , 'PF-PDS' (Projection-free primal-dual splitting method) or 'DR' (Douglas-Rachford-type splitting method)
             Default value : 'not specified', which means that the function :func:`choose_numerical_method` will choose it accordingly to the formulation
 
-        lambdas (numpy.ndarray) : list of lambdas for computinf lasso-path for cross validation on lambda.
-            Default value : None 
+        lambdas (numpy.ndarray) : list of rescaled lambdas for computing lasso-path for cross validation on lambda.
+            Default value : None, which means line space between 1 and lamin and Nlam points, with logarithm scale or not depending on logscale
+
+        Nlam (int) : number of points in the lambda-path if lambdas is still None
+            Default value : 80
+
+        lamin (float) : lambda minimum if lambdas is still None
+            Default value : 1e3
+
+        logscale (bool): when lambdas is set to None (default), this parameters tells if it should be set with log scale or not
+            Default value : True
 
         oneSE (bool) : if set to True, the selected lambda if computed with method 'one-standard-error'
             Default value : True
@@ -332,18 +358,28 @@ class CVparameters:
         self.seed = 0
         self.formulation = 'not specified'
         self.numerical_method = method
-
         self.Nsubset = 5  # Number of subsets used
         self.Nlam = 80
+        self.lamin = 1e3
+        self.logscale = True
         self.lambdas = None
         self.oneSE = True
 
     def __repr__(self): 
+        if not self.lambdas is None : 
+            self.Nlam = len(self.lambdas)
+            self.lamin = min(self.lambdas)
+            if self.logscale: typ = " with log-scale"
+            else : typ = " with linear-scale"
+        else :
+            typ = ' the list has been entered manually'
+
         string  = '\n     numerical_method : ' + str(self.numerical_method)
         string += '\n     one-SE method : ' + str(self.oneSE)
         string += '\n     Nsubset = ' + str(self.Nsubset)
-        string += '\n     lamin = ' + str(self.lambdas[-1])
-        string += '\n     Nlam = ' + str(len(self.lambdas))
+        string += '\n     lamin = ' + str(self.lamin)
+        string += '\n     Nlam = ' + str(self.Nlam)
+        string += typ
         
         
         return string
@@ -540,6 +576,11 @@ class solution_PATH:
         numerical_method = choose_numerical_method(param.numerical_method, 'PATH', param.formulation)
         param.numerical_method = numerical_method
         # Compute the solution and is the formulation is concomitant, it also compute sigma
+        if param.lambdas is None : 
+            if param.logscale : 
+                param.lambdas = np.array([param.lamin**(i/(param.Nlam-1)) for i in range(param.Nlam)])
+            else : 
+                np.linspace(1.,param.lamin,param.Nlam)
 
         out = pathlasso(matrices, lambdas=param.lambdas, n_active=param.n_active,
                                                 typ=name_formulation, meth=numerical_method, return_sigm=True,
@@ -560,12 +601,18 @@ class solution_PATH:
     def __repr__(self):
 
         string = "\n PATH COMPUTATION : "
+
         if len(self.BETAS[0])>100: 
             top = np.argpartition(np.mean(abs(np.array(self.BETAS)),axis=0), -100)[-100:]
         else : 
             top = np.arange(len(self.BETAS[0]))
 
-        affichage(self.BETAS[:,top], self.LAMBDAS, labels=self.label, naffichage=5,
+        if self.formulation.intercept : 
+            if 0 in top : top.remove(0)
+            string+="\n There is also an intercept.  "
+
+
+        affichage(self.BETAS[:,top], self.LAMBDAS, labels=self.label[top], naffichage=5,
                   title=PATH_beta_path["title"] + self.formulation.name(),xlabel=PATH_beta_path["xlabel"],ylabel=PATH_beta_path["ylabel"])
         if (type(self.save) == str): plt.savefig(self.save + 'Beta-path')
         plt.show()
@@ -592,7 +639,7 @@ class solution_CV:
         index_1SE (int) : index on xGraph of the selected lambda with 1-standard-error method
         lambda_min (float) : selected lambda without 1-standard-error method
         lambda_oneSE (float) : selected lambda with 1-standard-error method
-        beta (numpy.ndarray) : solution beta of classo at lambda_oneSE
+        beta (numpy.ndarray) : solution beta of classo at lambda_oneSE/lambda_min depending on CVparameters.oneSE
         sigma (float) : solution sigma of classo at lambda_oneSE when formulation is 'R2' or 'R4'
         selected_param (numpy.ndarray) : boolean arrays of size d with True when the variable is selected
         refit (numpy.ndarray) : solution beta after solving unsparse problem over the set of selected variables.
@@ -616,7 +663,11 @@ class solution_CV:
         param.numerical_method = numerical_method
 
 
-        if param.lambdas is None : param.lambdas = np.linspace(1.,1e-3,param.Nlam)
+        if param.lambdas is None : 
+            if param.logscale : 
+                param.lambdas = np.array([param.lamin**(i/(param.Nlam-1)) for i in range(param.Nlam)])
+            else : 
+                np.linspace(1.,param.lamin,param.Nlam)
 
         # Compute the solution and is the formulation is concomitant, it also compute sigma
         out, self.yGraph, self.standard_error, self.index_min, self.index_1SE = CV(matrices, param.Nsubset,
@@ -637,7 +688,7 @@ class solution_CV:
         else:
             self.beta = out
 
-        self.selected_param = abs(self.beta) > 1e-3  # boolean array, false iff beta_i =0
+        self.selected_param = abs(self.beta) > 1e-5  # boolean array, false iff beta_i =0
         self.refit = min_LS(matrices, self.selected_param, intercept=param.formulation.intercept)
         self.time = time() - t0
         self.save=False
@@ -647,14 +698,22 @@ class solution_CV:
 
         string = "\n CROSS VALIDATION : "
         d = len(self.refit)
-        if d>100: 
-            top = np.argpartition(abs(self.refit)+np.random.randn(d)*1e-5, -100)[-100:]
+        selected = self.selected_param[:]
+        if param.formulation.intercept : 
+            selected[0]=False
+            string+="\n Intercept : " + str(self.refit[0])
+
+        nb_select = sum(selected)
+        if nb_select>10 : 
+            top = np.argpartition(abs(self.refit[selected]), -10)[-10:]
             top = np.sort(top)
         else : 
-            top = np.arange(d)
+            top = np.arange(nb_select)
+
+
+        plt.bar(range(nb_select), self.refit[selected]), plt.title(CV_beta["title"]), plt.xlabel(CV_beta["xlabel"]),plt.ylabel(CV_beta["ylabel"])
+        plt.xticks(top,self.label[selected][top], rotation=90)
 
-        plt.bar(range(len(self.refit[top])), self.refit[top]), plt.title(CV_beta["title"]), plt.xlabel(CV_beta["xlabel"]),plt.ylabel(CV_beta["ylabel"])
-        plt.xticks(np.where(self.selected_param[top])[0],self.label[top][self.selected_param[top]], rotation=30)
         if(type(self.save)==str): plt.savefig(self.save)
         plt.show()
         self.graphic()
@@ -666,18 +725,44 @@ class solution_CV:
         string += "\n   Running time :  "  + str(round(self.time, 3)) + "s"
         return string
 
-    def graphic(self, ratio_mse_max=1.,save=False):
-        i_min, i_1SE = self.index_min, self.index_1SE
-        mse_max = ratio_mse_max * self.standard_error[i_min]
-        j = 0
-        while(j < i_1SE - 30  and self.yGraph[j] > mse_max) : j+=1
+    def graphic(self, se_max=10,save=None, logScale= False, errorevery=5):
+        ''' Method to plot the graphic showing mean squared error over along lambda path once cross validation is computed. 
 
-        plt.errorbar(self.xGraph[j:], self.yGraph[j:], self.standard_error[j:], label='mean over the k groups of data', errorevery = 10 )
-        plt.axvline(x=self.xGraph[i_min], color='k', label=r'$\lambda$ (min MSE)')
-        plt.axvline(x=self.xGraph[i_1SE],color='r',label=r'$\lambda$ (1SE) ')
-        plt.title(CV_graph["title"]), plt.xlabel(CV_graph["xlabel"]),plt.ylabel(CV_graph["ylabel"])
+        Args:
+            ratio_mse_max (float): float thanks to which the graphic will not show the lambdas from which MSE(lambda)> min(MSE) + ratio * Standard_error(lambda_min) .
+                this parameter is useful to plot a graph that zooms in the interesting part. 
+                Default value : 10
+            logScale (bool) : input that tells to plot the mean square error as a function of lambda, or log10(lambda)
+                Default value : False
+            errorevery (int) : parameter input of matplotlib.pyplot.errorbar that gives the frequency of the error bars appearence. 
+                Default value : 5
+            save (string) : path to the file where the figure should be saved. If None, then the figure will not be saved. 
+                Default Value : None
+
+        '''
+        
+        i_min, i_1SE = self.index_min, self.index_1SE
+        
+        jmin,jmax = 0,len(self.yGraph)-1
+        if not se_max is None : 
+            # jmin and jmax are the bounds on the xaxis to know where to zoom in in the lambda path
+            y_max = self.yGraph[i_min] + se_max * self.standard_error[i_min]
+            while(jmin < i_1SE and self.yGraph[jmin] > y_max) : jmin+=1
+            while(jmax > i_min and self.yGraph[jmax] > y_max) : jmax-=1
+
+        if logScale : 
+            plt.errorbar(np.log10(self.xGraph[jmin:jmax+1]), self.yGraph[jmin:jmax+1], self.standard_error[jmin:jmax+1], label='mean over the k groups of data', errorevery = errorevery )
+            plt.xlabel(r"$ \log_{10} \lambda / \lambda_{max}$")
+            plt.axvline(x=np.log10(self.xGraph[i_min]), color='k', label=r'$\lambda$ (min MSE)')
+            plt.axvline(x=np.log10(self.xGraph[i_1SE]),color='r',label=r'$\lambda$ (1SE) ')
+        else:
+            plt.errorbar(self.xGraph[jmin:], self.yGraph[jmin:], self.standard_error[jmin:], label='mean over the k groups of data', errorevery = errorevery )
+            plt.xlabel(r"$\lambda / \lambda_{max}$")
+            plt.axvline(x=self.xGraph[i_min], color='k', label=r'$\lambda$ (min MSE)')
+            plt.axvline(x=self.xGraph[i_1SE],color='r',label=r'$\lambda$ (1SE) ')
+        plt.title(CV_graph["title"]), plt.ylabel(CV_graph["ylabel"])
         plt.legend()
-        if(type(save)==str) : plt.savefig(save)
+        if not save is None: plt.savefig(save)
         plt.show()
 
 #Here, the main function used is stability ; from the file stability selection
@@ -740,8 +825,8 @@ class solution_StabSel:
         self.distribution = distribution
         self.distribution_path = distribution_path
         self.lambdas_path = lambdas
-        self.selected_param, self.to_label = selected_param(self.distribution, param.threshold,param.threshold_label)
-
+        self.selected_param = self.distribution > param.threshold
+        self.threshold_label = param.threshold_label
         self.threshold = param.threshold
         self.refit = min_LS(matrices, self.selected_param,  intercept=param.formulation.intercept)
         self.save1 = False
@@ -763,6 +848,7 @@ class solution_StabSel:
         else : 
             top = np.arange(d)
 
+        self.to_label = self.distribution > self.threshold_label
         D, Dpath, selected = self.distribution[top], self.distribution_path, np.array(self.selected_param)[top]
         unselected = [not i for i in selected]
         Dselected, Dunselected  = np.zeros(len(D)), np.zeros(len(D))
@@ -877,7 +963,7 @@ class solution_LAMfixed:
         else : 
             top = np.arange(d)
 
-        plt.bar(range(len(self.beta[top])), self.beta[top]), plt.title(LAM_beta["title"] + str(round(self.lam,3) ) ), plt.xlabel(LAM_beta["xlabel"]),plt.ylabel(LAM_beta["ylabel"])
+        plt.bar(range(len(top)), self.beta[top]), plt.title(LAM_beta["title"] + str(round(self.lam,3) ) ), plt.xlabel(LAM_beta["xlabel"]),plt.ylabel(LAM_beta["ylabel"])
         plt.xticks(np.where(self.selected_param[top])[0],self.label[top][self.selected_param[top]], rotation=30)
         
         if(type(self.save)==str): plt.savefig(self.save)
@@ -929,7 +1015,7 @@ def choose_numerical_method(method, model, formulation, StabSelmethod=None, lam=
 
         else:
             if not method in ['Path-Alg', 'DR', 'P-PDS', 'PF-PDS']:
-                if (lam > 0.1):
+                if (lam > 0.05):
                     return 'Path-Alg'
                 else:
                     return 'DR'
